"""
Constants for Lyra.
"""
from lyra.enums import Environment

PUBLIC_HEADERS = {"accept": "application/json", "content-type": "application/json"}

TEST_PRIVATE_KEY = "0xc14f53ee466dd3fc5fa356897ab276acbef4f020486ec253a23b0d1c3f89d4f4"

CONTRACTS = {
    Environment.TEST: {
        "BASE_URL": "https://api-demo.lyra.finance",
        "WS_ADDRESS": "wss://api-demo.lyra.finance/ws",
        "ACTION_TYPEHASH": '0x4d7a9f27c403ff9c0f19bce61d76d82f9aa29f8d6d4b0c5474607d9770d1af17',
        "DOMAIN_SEPARATOR": '0x9bcf4dc06df5d8bf23af818d5716491b995020f377d3b7b64c29ed14e3dd1105',
        "ETH_PERP_ADDRESS": '0x010e26422790C6Cb3872330980FAa7628FD20294',
        "BTC_PERP_ADDRESS": '0xAFB6Bb95cd70D5367e2C39e9dbEb422B9815339D',
        "ETH_OPTION_ADDRESS": '0xBcB494059969DAaB460E0B5d4f5c2366aab79aa1',
        "BTC_OPTION_ADDRESS": '0xAeB81cbe6b19CeEB0dBE0d230CFFE35Bb40a13a7',
        "TRADE_MODULE_ADDRESS": '0x87F2863866D85E3192a35A73b388BD625D83f2be',
        "STANDARD_RISK_MANAGER_ADDRESS": "0x28bE681F7bEa6f465cbcA1D25A2125fe7533391C",
        "BTC_PORTFOLIO_RISK_MANAGER_ADDRESS": "0xbaC0328cd4Af53d52F9266Cdbd5bf46720320A20",
        "ETH_PORTFOLIO_RISK_MANAGER_ADDRESS": "0xDF448056d7bf3f9Ca13d713114e17f1B7470DeBF",
        "CASH_ASSET": "0x6caf294DaC985ff653d5aE75b4FF8E0A66025928",
        "USDC_ASSET": "0xe80F2a02398BBf1ab2C9cc52caD1978159c215BD",
        "DEPOSIT_MODULE_ADDRESS": "0x43223Db33AdA0575D2E100829543f8B04A37a1ec",
        "TRANSFER_MODULE_ADDRESS": "0x0CFC1a4a90741aB242cAfaCD798b409E12e68926",
    },
    Environment.PROD: {
        "BASE_URL": "https://api.lyra.finance",
        "WS_ADDRESS": "wss://api.lyra.finance/ws",
        "ACTION_TYPEHASH": '0x4d7a9f27c403ff9c0f19bce61d76d82f9aa29f8d6d4b0c5474607d9770d1af17',
        "DOMAIN_SEPARATOR": '0xd96e5f90797da7ec8dc4e276260c7f3f87fedf68775fbe1ef116e996fc60441b',
        "ETH_PERP_ADDRESS": '0xAf65752C4643E25C02F693f9D4FE19cF23a095E3',
        "BTC_PERP_ADDRESS": '0xDBa83C0C654DB1cd914FA2710bA743e925B53086',
        "ETH_OPTION_ADDRESS": '0x4BB4C3CDc7562f08e9910A0C7D8bB7e108861eB4',
        "BTC_OPTION_ADDRESS": '0xd0711b9eBE84b778483709CDe62BacFDBAE13623',
        "TRADE_MODULE_ADDRESS": '0xB8D20c2B7a1Ad2EE33Bc50eF10876eD3035b5e7b',
        "STANDARD_RISK_MANAGER_ADDRESS": "0x28c9ddF9A3B29c2E6a561c1BC520954e5A33de5D",
        "BTC_PORTFOLIO_RISK_MANAGER_ADDRESS": "0x45DA02B9cCF384d7DbDD7b2b13e705BADB43Db0D",
        "ETH_PORTFOLIO_RISK_MANAGER_ADDRESS": "0xe7cD9370CdE6C9b5eAbCe8f86d01822d3de205A0",
        "CASH_ASSET": "0x57B03E14d409ADC7fAb6CFc44b5886CAD2D5f02b",
        "USDC_ASSET": "0x6879287835A86F50f784313dBEd5E5cCC5bb8481",
        "DEPOSIT_MODULE_ADDRESS": "0x9B3FE5E5a3bcEa5df4E08c41Ce89C4e3Ff01Ace3",
        "TRANSFER_MODULE_ADDRESS": "0x01259207A40925b794C8ac320456F7F6c8FE2636",
    },
}
